import json
import time
import zmq
import sys
sys.path.append("../../bpi_imports/")  # Make import search the 'bpi_import' directory also.
import bpi_communication as com


def read_arm_cal():
    """
    We assume that only one MCA is attached, and that the MCA Data Server is running.
    
    To check, see out_file = "../data/arm_cal.json"

    :param file_name: Read the calibration lookup tables from this file
    :return: None
    """

    mds_ip = "tcp://127.0.0.1:5507"
    mds_client = com.zmq_device(mds_ip, "client")

    cmd = {"type": "server_cmd", "name": "hello"}
    ret = mds_client.sar(cmd)
    sn_list = ret['sn_list']
    for sn in sn_list:
        print("Serial number:", sn)
    sn = sn_list[0]  # We only use one attached device here

    
    # Read back and store
    # Read arm_cal data.  Note that if int(arm_cal[63]) & 0x1 == 1,
    # (ie arm_cal_fields["lut_mode"] & 0x1 ==1) you cannot read back the data.
    cmd = {"type": "mca_cmd", "name": "arm_cal", "dir": "read", "sn": sn}
    ret = mds_client.sar(cmd)
    arm_cal = ret[sn]["fields"]

    out_file = "../data/arm_cal.json"
    with open(out_file, 'w') as fout:
        fout.write(json.dumps(arm_cal) + "\n")
    print("Output file: ", out_file)


read_arm_cal()
