import json
import time
import zmq
import sys
sys.path.append("../../bpi_imports/")  # Make import search the 'bpi_import' directory also.
import bpi_communication


def write_arm_ctrl(file_name):
    """
    We assume that only one MCA is attached, and that the MCA Data Server is running.
    Program the arm_ctrl structure in the MCA to program one or more parameters.
    The parameter names are the same as those shown in GUI, or the online documentation.

    :param file_name: Write the resulting arm_ctrl data to this file
    :return:
    """
    mds_client, sn = bpi_communication.connect_one_device()

    # Program the some members of the arm_ctrl structure;
    cmd = {"type": "mca_cmd", "name": "arm_ctrl", "dir": "rmw", "sn": sn,
           "data": {"fields": {"trigger_threshold": 0.02, "temp_ctrl": 1}, "user":{"histogram_run": 1}}}
    mds_client.sar(cmd)

    # Read back and store
    cmd = {"type": "mca_cmd", "name": "arm_ctrl", "dir": "read", "sn": sn}
    ret = mds_client.sar(cmd)
    arm_ctrl = ret[sn]
    with open(file_name, 'w') as fout:
        fout.write(json.dumps(arm_ctrl) + ",\n")


write_arm_ctrl(file_name="../data/arm_ctrl.json")
