import json
import time
import zmq
import sys
sys.path.append("../../bpi_imports/")  # Make import search the 'bpi_import' directory also.
import bpi_communication


def baseline_scan(par):
    """
    We assume that only one SiPM-1000 is attached, and that the MCA Data Server is running.
    This example is used with SiPM-based MCA.  It measures the ADC baseline as a function
    of applied operating voltage.  The baseline will change if there is significant
    dark current flowing through the SiPM array.

    :param file_name: Write the resulting arm_ctrl data to this file
    :return:
    """

    file_name = par["file_name"]
    ov_max = par["ov_max"]
    ov_min = par["ov_min"]
    n_steps = par["n_steps"]

    mds_client, sn = bpi_communication.connect_one_device()

    dov = (ov_max - ov_min)/n_steps

    with open(file_name, 'w') as fout:
        pass

    cmd_set_ov = {"type": "mca_cmd", "name": "arm_ctrl", "dir": "rmw", "sn": sn, "data": {"fields": {}}}
    cmd_read_status = {"type": "mca_cmd", "name": "arm_status", "dir": "read", "sn": sn}
    for n in range(n_steps+1):
        # Set new operating voltage
        ov = ov_min + n*dov
        cmd_set_ov["data"]["fields"]["cal_ov"] = ov
        mds_client.sar(cmd_set_ov)
        time.sleep(5)

        ret = mds_client.sar(cmd_read_status)
        fields = ret[sn]["fields"]
        results = {"ov": fields["voltage_target"], "baseline": fields["baseline"]}
        with open(file_name, 'a') as fout:
            fout.write(json.dumps(results) + "\n")

par = {
    "file_name": "../data/baseline_scan.dat",
    "ov_max": 36,
    "ov_min": 32,
    "n_steps": 20
}
baseline_scan(par)
