import json
import time
import sys
sys.path.append("../../bpi_imports/")  # Make import search the 'bpi_import' directory also.
import bpi_communication


def count_and_read(par):
    """
    We assume that only one device is attached, and that the device_server is running.
    Program the arm_ctrl structure in the device to count for 'dwell_time' seconds, then read the result.

    :param file_name: Write the arm_status data to this file after the read back.
    :return:
    """
    file_name = par["file_name"]
    dwell_time = par["dwell_time"]

    mds_client, sn = bpi_communication.connect_one_device()

    # Set the parameters for sample counting
    cmd = {"type": "mca_cmd", "name": "arm_ctrl", "dir": "rmw", "sn": sn,
           "data": {"fields": { "run_mode": 1, "run_time_sample": 10.0},
                    "user": {"clear_statistics": 1, "active_bank": 0}}}
    ret = mds_client.sar(cmd)

    time.sleep(dwell_time+1)  # Make sure the counting time has passed.

    # Read back data and store
    # In the output file look for the fields "run_time", "count_rate", and "count_rate_err".
    cmd = {"type": "mca_cmd", "name": "arm_status", "dir": "read", "sn": sn}
    ret = mds_client.sar(cmd)
    arm_status = ret[sn]
    with open(file_name, 'w') as fout:
        fout.write(json.dumps(arm_status) + "\n")

par = {
    "file_name": "../data/arm_status.json",
    "dwell_time": 10 # in seconds
}
count_and_read(par)
