import math
import json
import matplotlib.pyplot as plt
import sys
sys.path.append("../../bpi_imports/")  # Make import search the 'import' directory in the parent directory also.
import bpi_plots
import sample_strength_plot_controls as pc

def collect_probability_data(file_name):
    """
        Read the data file and plot sample signal strength.
        signal_strength = -log10(false alarm probability)
    """
    all_data = []
    with open(file_name, 'r') as fin:
        for line in fin:
            all_data += [json.loads(line)]

    num_records = len(all_data)
    y_prob_low = [-math.log10(all_data[n]["fields"]["bck_low_probability"]) for n in range(num_records)]
    y_prob = [-math.log10(all_data[n]["fields"]["bck_probability"]) for n in range(num_records)]
    y_prob_high = [-math.log10(all_data[n]["fields"]["bck_high_probability"]) for n in range(num_records)]

    x_data = [all_data[n]["fields"]["run_time_sample"] for n in range(num_records)]

    return x_data, [y_prob_low, y_prob, y_prob_high]


def collect_count_rate_data(file_name):
    """
        Read the 'sample_records.json' data file and plot difference count rates and errors
    """
    all_data = []
    with open(file_name, 'r') as fin:
        for line in fin:
            all_data += [json.loads(line)]

    num_records = len(all_data)
    y_diff_low = [all_data[n]["fields"]["roi_rate_diff"]-all_data[n]["fields"]["roi_rate_diff_err"] for n in range(num_records)]
    y_diff = [all_data[n]["fields"]["roi_rate_diff"] for n in range(num_records)]
    y_diff_high = [all_data[n]["fields"]["roi_rate_diff"]+all_data[n]["fields"]["roi_rate_diff_err"] for n in range(num_records)]

    x_data = [all_data[n]["fields"]["run_time_sample"] for n in range(num_records)]

    return x_data, [y_diff_high, y_diff, y_diff_low]



#file_name = "../data/sample_2022-08-05_15-50-11.dat"  # Background only
file_name = "../data/sample_2022-08-05_16-03-24.dat"  # Cs-137, 3.8uCi at 2.15m distance from 50mm NaI-detector
#file_name = "../data/sample_2022-08-05_16-51-09.dat"  # Cs-137, 3.8uCi at 1.45m distance from 50mm NaI-detector
x_data, y_data = collect_probability_data(file_name)
bpi_plots.line_plot_y_array(x_data, y_data, pc.prob_plot_ctrl)

x_data, y_data = collect_count_rate_data(file_name)
bpi_plots.line_plot_y_array(x_data, y_data, pc.diff_rate_plot_ctrl)


plt.show()
