#!/usr/bin/python
#
from __future__ import division
import time
import math
import json
import time
import sys
sys.path.append("../../bpi_imports/")  # Make import search the 'bpi_import' directory also.
import bpi_communication as com

version = 1.0

def acquire_pulses(par):
    """
    We assume that only one PMT-2000 is attached, and that the MCA Data Server is running.

    :param num_pulses: Number of pulses to acquire
    :param num_samples: Number of ADC samples per pulse
    :param file_name: Write the pulses to this file as a one-line json strings
    :param wait: >=0 => wait x seconds between start and read; <0 check until trace_done=1
    :return: None
    """
    num_pulses = par["num_pulses"]
    num_samples = par["num_samples"]
    file_name = par["file_name"]
    wait = par["wait"]

    # 127.0.0.1 is the same as localhost
    mds_client = com.zmq_device("tcp://127.0.0.1:5507", "client")  # Communicate with MDS

    # get serial number
    cmd = {"type": "server_cmd", "name": "hello"}
    ret = mds_client.sar(cmd)
    sn_list = ret["sn_list"]
    for sn in sn_list:
        print("Unique serial number:", sn)
    sn = sn_list[0]  # We just serve one device here

    # We need these three commands
    cmd_start = {"type": "mca_cmd", "name": "arm_ctrl",  "dir": "rmw", "sn": sn,
                 "data": {"fields": {}, "user":{"trace": 1, "clear_trace": 1,
                 "histogram_run": 0}}}
    cmd_read = {"type": "mca_cmd", "name": "arm_trace",  "dir": "read", "num_items": num_samples, "sn": sn }
    cmd_check = {"type": "mca_cmd", "name": "arm_status",  "dir": "read", "sn": sn }

    for num_pulse in range(num_pulses):
        mds_client.sar(cmd_start)

        if wait >= 0:
            time.sleep(wait)
        else:
            while True:
                ret_check = mds_client.sar(cmd_check)
                trace_done = ret_check[sn]["user"]["trace_done"]
                if trace_done:
                    break;

        ret_read = mds_client.sar(cmd_read)
        pulse = ret_read[sn]["registers"]

        with open(file_name, 'a') as fout:
            fout.write(json.dumps(pulse) + "\n")

par = {
    "num_pulses": 10,
    "num_samples": 256,
    "wait": -1,  # Sleep time between pulses, if >0
    "file_name": "../data/nai_pulses.dat"
}
acquire_pulses(par)
